@@ -141,7 +141,7 @@ class Url(NamedTuple):
             'fragment': None
         }
         # parse out the scheme, if present
-        if re.match('^\w+://', url_str):
+        if re.match(r'^\w+://', url_str):
             (scheme, url_str) = url_str.split('://', 1)
             parts['scheme'] = scheme.lower()
         # check for a path
@@ -171,7 +171,10 @@ class Url(NamedTuple):
         parts['fragment'] = parsed.fragment
         # normalize a bit and supply any defaults
         if not parts['scheme']:
-           parts['scheme'] = 'https'
+            if not parts['port'] or parts['port'] == 443:
+               parts['scheme'] = 'https'
+            else:
+               parts['scheme'] = 'http'
         if not parts['port']:
             if parts['scheme'] == 'https':
                 parts['port'] = 443
